#include <limits.h>
#include "handler.h"

/* WARC file id version 0.9 only */
#define WARC_ID "warc/0.9"
#define WARC_ID_LENGTH (sizeof(WARC_ID) - 1)

int
warc_handler_new(warc_handler_t *handler, char const *path_name)
{
    /* open file */
    handler->file_descr = open(path_name, O_RDONLY);
    if (handler->file_descr == -1) {
        return 0;
    }

    /* get file size */
    if (fstat(handler->file_descr, &handler->stat_info) == -1) {
        close(handler->file_descr);
        return 0;
    }

    /* map file */
    handler->base_prt = mmap(NULL, handler->stat_info.st_size, PROT_READ, MAP_PRIVATE, handler->file_descr, 0);
    handler->current_prt = handler->base_prt;
    if (handler->base_prt == MAP_FAILED) {
        close(handler->file_descr);
        return 0;
    }

    return 1;
}

int
warc_handler_del(warc_handler_t *handler)
{
    int ok = 1;

    /* unmap file */
    if (munmap((void *)handler->base_prt, handler->stat_info.st_size) == -1) {
        ok = 0;
    }

    /* close file */
    if (close(handler->file_descr) == -1) {
        ok = 0;
    }

    return ok;
}

int
warc_handler_next_raw_record(warc_handler_t *handler, warc_pointer_t *storage)
{
    char const *record_size_ptr;
    char *check;
    size_t to_process;
    long record_size;

    /* initialize the storage to the current position */
    storage->start = handler->current_prt;

    /* check EOF */
    to_process = handler->stat_info.st_size - (storage->start - handler->base_prt);
    if (to_process == 0) {
        return -1;
    }

    /* check for invalid records (according to the WARC id) */
    if (to_process < WARC_ID_LENGTH || strncmp(storage->start, WARC_ID, WARC_ID_LENGTH) != 0) {
        return 0;
    }

    /* skip the warc id */
    record_size_ptr = storage->start + WARC_ID_LENGTH;

    /* skip spaces */
    /* TODO check for premature EOF */
    while (isspace(*record_size_ptr)) {
        record_size_ptr++;
    }

    /* record size conversion */
    /* TODO check for premature EOF */
    record_size = strtol(record_size_ptr, &check, 10);
    if (!isspace(*check) || record_size <= 0 || record_size == LONG_MAX || record_size == LONG_MIN) {
        return 0;
    }

    /* jump to the end of the record and advance the pointer in the handler */
    /* TODO check for premature EOF */
    handler->current_prt = storage->end = storage->start + record_size;

    return 1;
}

int
warc_handler_next_record(warc_handler_t *handler, warc_record_t *record)
{
    int ret;

    /* read the next raw record */
    ret = warc_handler_next_raw_record(handler, &record->storage);
    if (ret != 1) {
        return ret;
    }

    /* dissect the fields */
    if (!warc_record_update(record)) {
        return 0;
    }

    return 1;
}
